﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using MessageBox = System.Windows.Forms.MessageBox;

namespace FaceChange
{
    internal class AppContext
    {
        private SynchronizationContext _synchronizationContext;
        // 串口类
        private SerialPort _serialPort;
        // Data
        private List<FaceInfo> _faces = new List<FaceInfo>();

        private static AppContext _appContext;

        private AppContext() { }

        public static AppContext GetAppContext()
        {
            if (_appContext == null)
            {
                _appContext = new AppContext();
                _appContext._faces = _appContext.FaceInfos;
                // 获取UI线程同步上下文
                _appContext._synchronizationContext = SynchronizationContext.Current;
            }
            return _appContext;
        }

        public SerialPort SerialPort
        {
            set
            {
                if (value == null)
                {
                    PortOpen = false;
                    MainWindow.Status.Content = "未连接";
                    MainWindow.Status.ToolTip = "设备未连接";
                    MainWindow.Status.Foreground = new SolidColorBrush(Colors.Red);
                }
                else
                {
                    PortOpen = true;
                    MainWindow.Status.Content = "已连接";
                    MainWindow.Status.ToolTip = "设备已连接";
                    MainWindow.Status.Foreground = new SolidColorBrush(Colors.Green);
                }
                _serialPort = value;
            }
            get
            {
                return _serialPort;
            }
        }

        // 打开串口标志位
        public bool PortOpen { set; get; }

        public List<FaceInfo> FaceInfos
        {
            set
            {
                _faces = value;
            }
            get
            {
                _faces.Clear();
                // 读取配置
                var prop = Properties.Settings.Default.Faces;
                if (prop == null || prop.Trim().Equals("")) return _faces;
                var faceStringList = prop.Split('|').ToList();
                foreach (var faceString in faceStringList)
                {
                    var fileName = faceString.Substring(
                        faceString.LastIndexOf(@"\", StringComparison.Ordinal) + 1);
                    try
                    {
                        _faces.Add(new FaceInfo(
                            new BitmapImage(new Uri(faceString)),
                            fileName,
                            faceString.Replace(fileName, @"")));
                    }
                    catch (Exception exception)
                    {
                        Console.WriteLine(exception);
                    }
                }
                return _faces;
            }
        }

        private int _lastIndex = -1;
        private int GetNoRepeatRandom(int min, int max)
        {
            if (max - min <= 1)
            {
                return min;
            }
            var r = new Random();
            var i = r.Next(min, max);
            while (i == _lastIndex)
            {
                i = r.Next(min, max);
            }
            _lastIndex = i;
            return i;
        }

        public void DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                // Comm.BytesToRead 中为要读入的字节长度
                var len = _serialPort.BytesToRead;
                var readBuffer = new byte[len];
                // 将数据读入缓存
                _serialPort.Read(readBuffer, 0, len);
                // 处理 readBuffer 中的数据，自定义处理过程
                var msg = Encoding.Default.GetString(readBuffer, 0, len);
                Console.Write(msg);
                // 委托执行
                var index = GetNoRepeatRandom(0, _faces.Count);
                ThreadProcSafePost(index);
            }
            catch (Exception ex)
            {
                MessageBox.Show("接收数据异常！具体原因：" + ex.Message, "错误提示");
            }
        }

        private void ThreadProcSafePost(int text)
        {
            _synchronizationContext.Post(SetTextSafePost, text);
        }

        private void SetTextSafePost(object index)
        {
            if (_faces.Count == 0)
            {
                MessageBox.Show("请先在设置页添加资源", "提示", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MainWindow.FaceName.Content = _faces[(int)index].Name;
                MainWindow.FaceSrc.Source = _faces[(int)index].Face;
                Console.WriteLine(index);
            }
        }
    }
}
